@@ -511,6 +511,7 @@ class LudwigModel:
                 # auto tune batch size
                 if self.config[TRAINER][BATCH_SIZE] == AUTO or self.config[TRAINER][EVAL_BATCH_SIZE] == AUTO:
                     # TODO (ASN): add support for substitute_with_max parameter
+                    # TODO(travis): detect train and eval batch sizes separately (enable / disable gradients)
                     tuned_batch_size = trainer.tune_batch_size(self.config, training_set, random_seed=random_seed)
 
                     # TODO(travis): pass these in as args to trainer when we call train,
@@ -519,7 +520,7 @@ class LudwigModel:
                         self.config[TRAINER][BATCH_SIZE] = tuned_batch_size
                         trainer.batch_size = tuned_batch_size
 
-                    if self.config[TRAINER][EVAL_BATCH_SIZE] == AUTO:
+                    if self.config[TRAINER][EVAL_BATCH_SIZE] in {AUTO, None}:
                         self.config[TRAINER][EVAL_BATCH_SIZE] = tuned_batch_size
                         trainer.eval_batch_size = tuned_batch_size
 
