@@ -16,7 +16,7 @@
 import logging
 from abc import ABC
 from functools import lru_cache
-from typing import Dict
+from typing import Any, Dict
 
 import torch
 from torch.nn import Linear, ModuleList
@@ -39,7 +39,7 @@ from ludwig.schema.combiners import (
     TabTransformerCombinerConfig,
     TransformerCombinerConfig,
 )
-from ludwig.schema.combiners.utils import combiner_registry, register_combiner
+from ludwig.schema.combiners.utils import combiner_registry, get_combiner_jsonschema, register_combiner
 from ludwig.utils.misc_utils import get_from_registry
 from ludwig.utils.torch_utils import LudwigModule, sequence_length_3D
 from ludwig.utils.torch_utils import sequence_mask as torch_sequence_mask
@@ -56,6 +56,10 @@ def get_combiner_class(combiner_type: str):
     return get_from_registry(combiner_type, combiner_registry)
 
 
+def get_combiner_schema() -> Dict[str, Any]:
+    return get_combiner_jsonschema()
+
+
 # super class to house common properties
 class Combiner(LudwigModule, ABC):
     def __init__(self, input_features: Dict[str, "InputFeature"]):
@@ -187,7 +191,7 @@ class SequenceConcatCombiner(Combiner):
     def concatenated_shape(self) -> torch.Size:
         # computes the effective shape of the input tensor after combining
         # all the encoder outputs
-        # determine sequence size by finding the first sequence tensor
+        # determine max sequence length by finding the first sequence tensor
         # assume all the sequences are of the same size, if not true
         # this will be caught during processing
         seq_size = None
@@ -341,7 +345,7 @@ class SequenceCombiner(Combiner):
     def concatenated_shape(self) -> torch.Size:
         # computes the effective shape of the input tensor after combining
         # all the encoder outputs
-        # determine sequence size by finding the first sequence tensor
+        # determine max sequence length by finding the first sequence tensor
         # assume all the sequences are of the same size, if not true
         # this will be caught during processing
         seq_size = None
@@ -473,8 +477,8 @@ class TransformerCombiner(Combiner):
         if self.reduce_output is None:
             self.supports_masking = True
 
-        # sequence size for Transformer layer is number of input features
-        self.sequence_size = len(self.input_features)
+        # max sequence length for Transformer layer is number of input features
+        self.max_sequence_length = len(self.input_features)
 
         logger.debug("  Projectors")
         self.projectors = ModuleList(
@@ -491,7 +495,7 @@ class TransformerCombiner(Combiner):
         logger.debug("  TransformerStack")
         self.transformer_stack = TransformerStack(
             input_size=config.hidden_size,
-            sequence_size=self.sequence_size,
+            max_sequence_length=self.max_sequence_length,
             hidden_size=config.hidden_size,
             num_heads=config.num_heads,
             output_size=config.transformer_output_size,
@@ -634,7 +638,7 @@ class TabTransformerCombiner(Combiner):
         logger.debug("  TransformerStack")
         self.transformer_stack = TransformerStack(
             input_size=config.hidden_size,
-            sequence_size=len(self.embeddable_features),
+            max_sequence_length=len(self.embeddable_features),
             hidden_size=config.hidden_size,
             # todo: can we just use projector_size? # hidden_size,
             num_heads=config.num_heads,
