@@ -14,6 +14,7 @@
 # limitations under the License.
 # ==============================================================================
 import logging
+from typing import Any, Dict, List
 
 import numpy as np
 import torch
@@ -22,6 +23,7 @@ from ludwig.constants import COLUMN, FILL_WITH_CONST, H3, MISSING_VALUE_STRATEGY
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature
 from ludwig.utils.h3_util import h3_to_components
 from ludwig.utils.misc_utils import set_default_value
+from ludwig.utils.types import TorchscriptPreprocessingInput
 
 logger = logging.getLogger(__name__)
 
@@ -30,6 +32,39 @@ H3_VECTOR_LENGTH = MAX_H3_RESOLUTION + 4
 H3_PADDING_VALUE = 7
 
 
+class _H3Preprocessing(torch.nn.Module):
+    def __init__(self, metadata: Dict[str, Any]):
+        super().__init__()
+        self.max_h3_resolution = MAX_H3_RESOLUTION
+        self.h3_padding_value = H3_PADDING_VALUE
+        self.computed_fill_value = float(metadata["preprocessing"]["computed_fill_value"])
+
+    def forward(self, v: TorchscriptPreprocessingInput) -> torch.Tensor:
+        if torch.jit.isinstance(v, List[torch.Tensor]):
+            v = torch.stack(v)
+
+        if not torch.jit.isinstance(v, torch.Tensor):
+            raise ValueError(f"Unsupported input: {v}")
+
+        v = torch.nan_to_num(v, nan=self.computed_fill_value)
+        v = v.long()
+
+        outputs: List[torch.Tensor] = []
+        for v_i in v:
+            components = h3_to_components(v_i)
+            header: List[int] = [
+                components.mode,
+                components.edge,
+                components.resolution,
+                components.base_cell,
+            ]
+            cells_padding: List[int] = [self.h3_padding_value] * (self.max_h3_resolution - len(components.cells))
+            output = torch.tensor(header + components.cells + cells_padding, dtype=torch.uint8)
+            outputs.append(output)
+
+        return torch.stack(outputs)
+
+
 class H3FeatureMixin(BaseFeatureMixin):
     @staticmethod
     def type():
@@ -53,8 +88,11 @@ class H3FeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def cast_column(column, backend):
-        # todo: add cast to int64
-        return column
+        try:
+            return column.astype(int)
+        except ValueError:
+            logging.warning("H3Feature could not be read as int directly. Reading as float and converting to int.")
+            return column.astype(float).astype(int)
 
     @staticmethod
     def get_feature_meta(column, preprocessing_parameters, backend):
@@ -63,9 +101,9 @@ class H3FeatureMixin(BaseFeatureMixin):
     @staticmethod
     def h3_to_list(h3_int):
         components = h3_to_components(h3_int)
-        header = [components["mode"], components["edge"], components["resolution"], components["base_cell"]]
-        cells_padding = [H3_PADDING_VALUE] * (MAX_H3_RESOLUTION - len(components["cells"]))
-        return header + components["cells"] + cells_padding
+        header = [components.mode, components.edge, components.resolution, components.base_cell]
+        cells_padding = [H3_PADDING_VALUE] * (MAX_H3_RESOLUTION - len(components.cells))
+        return header + components.cells + cells_padding
 
     @staticmethod
     def add_feature_data(
@@ -121,3 +159,7 @@ class H3InputFeature(H3FeatureMixin, InputFeature):
     @staticmethod
     def populate_defaults(input_feature):
         set_default_value(input_feature, TIED, None)
+
+    @staticmethod
+    def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+        return _H3Preprocessing(metadata)
