@@ -14,6 +14,7 @@
 # limitations under the License.
 # ==============================================================================
 import logging
+from functools import partial
 from typing import Any, Dict
 
 import torch
@@ -40,8 +41,13 @@ from ludwig.constants import (
 )
 from ludwig.encoders.registry import get_encoder_cls
 from ludwig.features.base_feature import BaseFeatureMixin, OutputFeature
-from ludwig.features.feature_utils import compute_sequence_probability
-from ludwig.features.sequence_feature import _SequencePreprocessing, SequenceInputFeature, SequenceOutputFeature
+from ludwig.features.feature_utils import compute_sequence_probability, compute_token_probabilities
+from ludwig.features.sequence_feature import (
+    _SequencePostprocessing,
+    _SequencePreprocessing,
+    SequenceInputFeature,
+    SequenceOutputFeature,
+)
 from ludwig.utils.math_utils import softmax
 from ludwig.utils.misc_utils import set_default_values
 from ludwig.utils.strings_utils import (
@@ -97,7 +103,7 @@ class TextFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def cast_column(column, backend):
-        return column
+        return column.astype(str)
 
     @staticmethod
     def feature_meta(column, preprocessing_parameters, backend):
@@ -134,7 +140,6 @@ class TextFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def get_feature_meta(column, preprocessing_parameters, backend):
-        column = column.astype(str)
         tf_meta = TextFeatureMixin.feature_meta(column, preprocessing_parameters, backend)
         (
             idx2str,
@@ -191,7 +196,7 @@ class TextFeatureMixin(BaseFeatureMixin):
         feature_config, input_df, proc_df, metadata, preprocessing_parameters, backend, skip_save_processed_input
     ):
         proc_df[feature_config[PROC_COLUMN]] = TextFeatureMixin.feature_data(
-            input_df[feature_config[COLUMN]].astype(str),
+            input_df[feature_config[COLUMN]],
             metadata[feature_config[NAME]],
             preprocessing_parameters,
             backend,
@@ -345,26 +350,29 @@ class TextOutputFeature(TextFeatureMixin, SequenceOutputFeature):
         probs_col = f"{self.feature_name}_{PROBABILITIES}"
         prob_col = f"{self.feature_name}_{PROBABILITY}"
         if probs_col in result:
-
+            # currently does not return full probabilties because usually it is huge:
+            # dataset x length x classes
+            # TODO: add a mechanism for letting the user decide to save it
+            result[probs_col] = backend.df_engine.map_objects(result[probs_col], compute_token_probabilities)
             result[prob_col] = backend.df_engine.map_objects(
                 result[probs_col],
-                compute_sequence_probability,
+                partial(
+                    compute_sequence_probability,
+                    max_sequence_length=metadata["max_sequence_length"],
+                    return_log_prob=True,
+                ),
             )
 
-            # commenting probabilities out because usually it is huge:
-            # dataset x length x classes
-            # todo: add a mechanism for letting the user decide to save it
-            # result[probs_col] = backend.df_engine.map_objects(
-            #     result[probs_col],
-            #     lambda prob: np.amax(prob, axis=-1),
-            # )
-
         lengths_col = f"{self.feature_name}_{LENGTHS}"
         if lengths_col in result:
             del result[lengths_col]
 
         return result
 
+    @staticmethod
+    def create_postproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
+        return _SequencePostprocessing(metadata)
+
     @staticmethod
     def populate_defaults(output_feature):
         SequenceOutputFeature.populate_defaults(output_feature)
